import 'dart:async';

import 'package:analyzer/dart/element/element.dart';
import 'package:analyzer/dart/element/type.dart';
import 'package:build/build.dart';
import 'package:json_serializable_dart/serializable_json.dart';
import 'package:source_gen/source_gen.dart';

import 'serializable_json.dart';

/// Generates serializable classes from classes annotated with `@SerializableJson()` or `@serializableJson`
class SerializableGenerator extends GeneratorForAnnotation<SerializableJson> {
  const SerializableGenerator();

  @override
  Future<String> generateForAnnotatedElement(
      Element element, ConstantReader cr, BuildStep buildStep) async {
    if (element is! ClassElement) return '';
    final className = element.name;
    final overlayName = cr.read('methodOverlayName').isNull
        ? element.name
        : cr.read('methodOverlayName').stringValue;
    final nullSafe =
        cr.read('nullSafe').isNull ? true : cr.read('nullSafe').boolValue;

    var fields = element.fields;
    var res =
        'void _\$${overlayName}FromJson($className instance, Map<String, dynamic> json) {\n'
        '${fields.map((s) => 'instance.${s.displayName}=json[\'${s.displayName}\'] ${nullSafe ? 'as ${s.type.getDisplayString(withNullability: nullSafe)}' : ''};').join('\n')}'
        '}\n\n\n'
        'Map<String, dynamic> _\$${overlayName}ToJson($className instance) =>\n'
        '<String, dynamic>{\n${fields.map((s) => '\'${s.displayName}\': instance.${s.displayName},').join('\n')}};';
    return res;
  }
}
